#include "defs.h"
#include "RealFuncs.h"
#include "RealConstants.h"
#include "RealObject.h"

#include <stdlib.h>
#include <assert.h>

namespace RealLib {

RealInternalConstant::RealInternalConstant(RealObject *pValue)
	: m_pVal(pValue)
{ 
	assert(pValue); 
	pValue->AddRef(); 
	pValue->AddRef(); 
	// AddRef twice ensures GetEstimate will cache the value
}

RealInternalConstant::~RealInternalConstant()
{ 
	m_pVal->Release(); 
	m_pVal->Release(); 
}

RealInternalConstant& RealInternalConstant::operator = (const RealInternalConstant &rhs)
{
	m_pVal->Release();
	m_pVal->Release();
	m_pVal = rhs.m_pVal;
	m_pVal->AddRef();
	m_pVal->AddRef();
	return *this;
}

Estimate RealInternalConstant::GetEstimate(u32 prec)
{ 
	// constants will often be touched by interested parties
	// do not delete the estimate
	++m_pVal->m_EstimateRefs;
	return m_pVal->GetEstimate(prec); 
}

// constants storage

RealConstants::RealConstants()
: rpi(new RealNullary(RealLib::rpi)),
  pi(new RealNullary(RealLib::pi)),
  ln2(new RealNullary(RealLib::ln2)),
  ln10(new RealUnary(RealLib::ln, new RealFromDouble(10.0)))
{
}

RealConstants Constants;

} // namespace