/*

  RealConstants.h

  This file defines the class RealConstants, and the object Constants.
  It is used to hold frequently used internal constants, and as a
  complement to the frontend.

*/

#ifndef FILE_REAL_CONSTANTS_H
#define FILE_REAL_CONSTANTS_H

#include "real.h"

namespace RealLib {

class RealObject;
class Estimate;

// class RealInternalConstant: the wrapper around the constant object.
// in function it is very much like class Real, but is simpler and
// takes care to not allow an estimate to be destroyed, unlike Reals
// which use reference counts.

class RealInternalConstant {
public:
	RealObject *m_pVal;

	RealInternalConstant(RealObject *pValue);
	~RealInternalConstant();
	
	// not needed
	RealInternalConstant& operator = (const RealInternalConstant &rhs);

	// the important function, get the value of the constant
	Estimate GetEstimate(u32 prec);

};

// class RealConstants hold the actual constants

class RealConstants {
public:
	RealInternalConstant rpi;
	RealInternalConstant pi;
	RealInternalConstant ln2;
	RealInternalConstant ln10;

	RealConstants();
};

// and this is its instantiation
extern RealConstants Constants;

} // namespace

#endif